@@ -3,7 +3,7 @@
  */
 
 /*
- * Copyright (C) 2016 Oracle Corporation
+ * Copyright (C) 2016-2019 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -23,9 +23,11 @@
  * terms and conditions of either the GPL or the CDDL or both.
  */
 
-
-#ifndef ___iprt_win_ntdddisk_h___
-#define ___iprt_win_ntdddisk_h___
+#ifndef IPRT_INCLUDED_win_ntdddisk_h
+#define IPRT_INCLUDED_win_ntdddisk_h
+#ifndef RT_WITHOUT_PRAGMA_ONCE
+# pragma once
+#endif
 
 #pragma warning(push)
 #pragma warning(disable:4668) /* ntdddisk.h(137) : warning C4668: 'NTDDI_WIN2003' is not defined as a preprocessor macro, replacing with '0' for '#if/#elif' */
@@ -34,5 +36,5 @@
 
 #pragma warning(pop)
 
-#endif
+#endif /* !IPRT_INCLUDED_win_ntdddisk_h */
 
