@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2013-2017 Oracle Corporation
+ * Copyright (C) 2013-2019 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -19,6 +19,7 @@
 /*********************************************************************************************************************************
 *   Header Files                                                                                                                 *
 *********************************************************************************************************************************/
+#include <stdio.h>
 #define LOG_GROUP LOG_GROUP_DEV_VMSVGA
 #include <VBox/vmm/pdmdev.h>
 #include <VBox/err.h>
@@ -290,11 +291,7 @@ static int vmsvga3dSurfaceUpdateHeapBuffers(PVMSVGA3DSTATE pState, PVMSVGA3DSURF
             PVMSVGA3DMIPMAPLEVEL pMipmapLevel = &pSurface->pMipmapLevels[iFace * pSurface->faces[0].numMipLevels];
             for (uint32_t i = 0; i < pSurface->faces[iFace].numMipLevels; i++, pMipmapLevel++)
             {
-#ifdef VMSVGA3D_DIRECT3D
-                if (pSurface->u.pSurface)
-#else
-                if (pSurface->oglId.texture != OPENGL_INVALID_ID)
-#endif
+                if (VMSVGA3DSURFACE_HAS_HW_SURFACE(pSurface))
                 {
                     Assert(pMipmapLevel->cbSurface);
                     Assert(pMipmapLevel->cbSurface == pMipmapLevel->cbSurfacePlane * pMipmapLevel->mipmapSize.depth);
@@ -449,11 +446,9 @@ static int vmsvga3dSurfaceUpdateHeapBuffers(PVMSVGA3DSTATE pState, PVMSVGA3DSURF
                     /*
                      * OpenGL specifics.
                      */
-                    switch (fSwitchFlags)
+                    switch (pSurface->enmOGLResType)
                     {
-                        case SVGA3D_SURFACE_HINT_TEXTURE:
-                        case SVGA3D_SURFACE_HINT_RENDERTARGET:
-                        case SVGA3D_SURFACE_HINT_TEXTURE | SVGA3D_SURFACE_HINT_RENDERTARGET:
+                        case VMSVGA3D_OGLRESTYPE_TEXTURE:
                         {
                             GLint activeTexture;
                             glGetIntegerv(GL_TEXTURE_BINDING_2D, &activeTexture);
@@ -481,9 +476,7 @@ static int vmsvga3dSurfaceUpdateHeapBuffers(PVMSVGA3DSTATE pState, PVMSVGA3DSURF
                             break;
                         }
 
-                        case SVGA3D_SURFACE_HINT_VERTEXBUFFER | SVGA3D_SURFACE_HINT_INDEXBUFFER:
-                        case SVGA3D_SURFACE_HINT_VERTEXBUFFER:
-                        case SVGA3D_SURFACE_HINT_INDEXBUFFER:
+                        case VMSVGA3D_OGLRESTYPE_BUFFER:
                         {
                             pState->ext.glBindBuffer(GL_ARRAY_BUFFER, pSurface->oglId.buffer);
                             VMSVGA3D_CHECK_LAST_ERROR(pState, pContext);
@@ -830,7 +823,7 @@ void vmsvga3dAsciiPrint(PFMVMSVGAASCIIPRINTLN pfnPrintLine, void *pvUser, void c
     /*
      * Skip stuff we can't or won't need to handle.
      */
-    if (!cx || !cy)
+    if (!cx || !cy || !cchMaxX || !cchMaxY)
         return;
     switch (enmFormat)
     {
@@ -1545,13 +1538,11 @@ static void vmsvga3dInfoContextWorkerOne(PCDBGFINFOHLP pHlp, PVMSVGA3DCONTEXT pC
 
 #else
     pHlp->pfnPrintf(pHlp, "window:                  %p\n", pContext->window);
-    pHlp->pfnPrintf(pHlp, "fMapped:                 %RTbool\n", pContext->fMapped);
     if (pContext->window)
         vmsvga3dInfoHostWindow(pHlp, (uintptr_t)pContext->window);
     pHlp->pfnPrintf(pHlp, "glxContext:              %p\n", pContext->glxContext);
 
 #endif
-    pHlp->pfnPrintf(pHlp, "sidRenderTarget:         %#x\n", pContext->sidRenderTarget);
 
     for (uint32_t i = 0; i < RT_ELEMENTS(pContext->aSidActiveTextures); i++)
         if (pContext->aSidActiveTextures[i] != SVGA3D_INVALID_ID)
@@ -1812,6 +1803,174 @@ static DECLCALLBACK(int) vmsvga3dInfoSharedObjectCallback(PAVLU32NODECORE pNode,
 #endif /* VMSVGA3D_DIRECT3D */
 
 
+static int vmsvga3dInfoBmpWrite(const char *pszFilename, const void *pvBits, int w, int h, uint32_t cbPixel, uint32_t u32Mask)
+{
+#ifdef RT_OS_WINDOWS
+    if (   cbPixel != 4
+        && cbPixel != 2
+        && cbPixel != 1)
+        return VERR_NOT_SUPPORTED;
+
+    /* Always write BGRX bitmap for now. */
+    const int cbBitmap = w * h * 4;
+
+    FILE *f = fopen(pszFilename, "wb");
+    if (!f)
+        return VERR_FILE_NOT_FOUND;
+
+    if (cbPixel == 4)
+    {
+        BITMAPV4HEADER bh;
+        RT_ZERO(bh);
+        bh.bV4Size          = sizeof(bh);
+        bh.bV4Width         = w;
+        bh.bV4Height        = -h;
+        bh.bV4Planes        = 1;
+        bh.bV4BitCount      = 32;
+        bh.bV4V4Compression = BI_BITFIELDS;
+        bh.bV4SizeImage     = cbBitmap;
+        bh.bV4XPelsPerMeter = 2835;
+        bh.bV4YPelsPerMeter = 2835;
+        // bh.bV4ClrUsed       = 0;
+        // bh.bV4ClrImportant  = 0;
+        bh.bV4RedMask       = 0x00ff0000;
+        bh.bV4GreenMask     = 0x0000ff00;
+        bh.bV4BlueMask      = 0x000000ff;
+        bh.bV4AlphaMask     = 0xff000000;
+        bh.bV4CSType        = LCS_WINDOWS_COLOR_SPACE;
+        // bh.bV4Endpoints     = {0};
+        // bh.bV4GammaRed      = 0;
+        // bh.bV4GammaGreen    = 0;
+        // bh.bV4GammaBlue     = 0;
+
+        BITMAPFILEHEADER bf;
+        bf.bfType = 'MB';
+        bf.bfSize = sizeof(bf) + sizeof(bh) + cbBitmap;
+        bf.bfReserved1 = 0;
+        bf.bfReserved2 = 0;
+        bf.bfOffBits = sizeof(bf) + sizeof(bh);
+
+        fwrite(&bf, 1, sizeof(bf), f);
+        fwrite(&bh, 1, sizeof(bh), f);
+    }
+    else
+    {
+        BITMAPFILEHEADER bf;
+        bf.bfType = 'MB';
+        bf.bfSize = sizeof(BITMAPFILEHEADER) + sizeof(BITMAPINFOHEADER) + cbBitmap;
+        bf.bfReserved1 = 0;
+        bf.bfReserved2 = 0;
+        bf.bfOffBits = sizeof(BITMAPFILEHEADER) + sizeof(BITMAPINFOHEADER);
+
+        BITMAPINFOHEADER bi;
+        bi.biSize = sizeof(bi);
+        bi.biWidth = w;
+        bi.biHeight = -h;
+        bi.biPlanes = 1;
+        bi.biBitCount = 32;
+        bi.biCompression = 0;
+        bi.biSizeImage = cbBitmap;
+        bi.biXPelsPerMeter = 0;
+        bi.biYPelsPerMeter = 0;
+        bi.biClrUsed = 0;
+        bi.biClrImportant = 0;
+
+        fwrite(&bf, 1, sizeof(bf), f);
+        fwrite(&bi, 1, sizeof(bi), f);
+    }
+
+    if (cbPixel == 4)
+    {
+        const uint32_t *s = (uint32_t *)pvBits;
+        int i;
+        for (i = 0; i < w * h; ++i)
+        {
+            const uint32_t u32 = *s++;
+            uint32_t u = u32 & u32Mask;
+            fwrite(&u, 1, 4, f);
+        }
+    }
+    else if (cbPixel == 2)
+    {
+        const uint16_t *s = (uint16_t *)pvBits;
+        int i;
+        for (i = 0; i < w * h; ++i)
+        {
+            const uint16_t u16 = *s++;
+            uint32_t u32 = u16;
+            uint32_t u = u32 & u32Mask;
+            fwrite(&u, 1, 4, f);
+        }
+    }
+    else if (cbPixel == 1)
+    {
+        const uint8_t *s = (uint8_t *)pvBits;
+        int i;
+        for (i = 0; i < w * h; ++i)
+        {
+            const uint8_t u8 = *s++;
+            uint32_t u32 = u8 * 0x10000 + u8 * 0x100 + u8;
+            uint32_t u = u32 & u32Mask;
+            fwrite(&u, 1, 4, f);
+        }
+    }
+
+    fclose(f);
+
+    return VINF_SUCCESS;
+#else /* !RT_OS_WINDOWS */
+    RT_NOREF6(pszFilename, pvBits, w, h, cbPixel, u32Mask);
+    return VERR_NOT_SUPPORTED;
+#endif
+}
+
+void vmsvga3dInfoSurfaceToBitmap(PCDBGFINFOHLP pHlp, PVMSVGA3DSURFACE pSurface,
+                                 const char *pszPath, const char *pszNamePrefix, const char *pszNameSuffix)
+{
+    static volatile uint32_t sSeq = 0;
+
+    if (!pSurface->pMipmapLevels[0].pSurfaceData)
+        return;
+
+    const uint32_t u32Seq = ASMAtomicIncU32(&sSeq);
+
+    char szFilepath[4096];
+    RTStrPrintf(szFilepath, sizeof(szFilepath),
+                "%s\\%s-%u-sid%u%s.bmp",
+                pszPath, pszNamePrefix, u32Seq, pSurface->id, pszNameSuffix);
+
+    const uint32_t cbPixel = vmsvga3dSurfaceFormatSize(pSurface->format, NULL, NULL);
+    int rc = vmsvga3dInfoBmpWrite(szFilepath,
+                                  pSurface->pMipmapLevels[0].pSurfaceData,
+                                  pSurface->pMipmapLevels[0].mipmapSize.width,
+                                  pSurface->pMipmapLevels[0].mipmapSize.height,
+                                  cbPixel, 0xFFFFFFFF);
+    if (RT_SUCCESS(rc))
+    {
+        Log(("Bitmap: %s\n", szFilepath));
+        if (pHlp)
+            pHlp->pfnPrintf(pHlp, "Bitmap: %s\n", szFilepath);
+    }
+    else
+    {
+        Log(("Bitmap: %s %Rrc\n", szFilepath, rc));
+        if (pHlp)
+            pHlp->pfnPrintf(pHlp, "Bitmap: %s %Rrc\n", szFilepath, rc);
+    }
+
+#if 0
+    /* Alpha channel alone. */
+    RTStrPrintf(szFilepath, sizeof(szFilepath),
+                "%s\\%s-%u-sid%u%s-a.bmp",
+                pszPath, pszNamePrefix, u32Seq, pSurface->id, pszNameSuffix);
+    vmsvga3dInfoBmpWrite(szFilepath,
+                         pSurface->pMipmapLevels[0].pSurfaceData,
+                         pSurface->pMipmapLevels[0].mipmapSize.width,
+                         pSurface->pMipmapLevels[0].mipmapSize.height,
+                         cbPixel, 0xFF000000);
+#endif
+}
+
 static void vmsvga3dInfoSurfaceWorkerOne(PCDBGFINFOHLP pHlp, PVMSVGA3DSURFACE pSurface,
                                          bool fVerbose, uint32_t cxAscii, bool fInvY)
 {
@@ -1918,7 +2077,7 @@ static void vmsvga3dInfoSurfaceWorkerOne(PCDBGFINFOHLP pHlp, PVMSVGA3DSURFACE pS
 }
 
 
-void vmsvga3dInfoSurfaceWorker(PVGASTATE pThis, PCDBGFINFOHLP pHlp, uint32_t sid, bool fVerbose, uint32_t cxAscii, bool fInvY)
+void vmsvga3dInfoSurfaceWorker(PVGASTATE pThis, PCDBGFINFOHLP pHlp, uint32_t sid, bool fVerbose, uint32_t cxAscii, bool fInvY, const char *pszBitmapPath)
 {
     /* Warning! This code is currently racing papSurfaces reallocation! */
     /* Warning! This code is currently racing papSurfaces reallocation! */
@@ -1939,6 +2098,8 @@ void vmsvga3dInfoSurfaceWorker(PVGASTATE pThis, PCDBGFINFOHLP pHlp, uint32_t sid
                     if (fVerbose)
                         vmsvga3dSurfaceUpdateHeapBuffersOnFifoThread(pThis, sid);
                     vmsvga3dInfoSurfaceWorkerOne(pHlp, pSurface, fVerbose, cxAscii, fInvY);
+                    if (pszBitmapPath && *pszBitmapPath)
+                        vmsvga3dInfoSurfaceToBitmap(pHlp, pSurface, pszBitmapPath, "info", "");
                     return;
                 }
             }
