@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2013-2017 Oracle Corporation
+ * Copyright (C) 2013-2019 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -15,26 +15,30 @@
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
  */
 
-#ifdef VBOX_WITH_PRECOMPILED_HEADERS
-# include <precomp.h>
-#else  /* !VBOX_WITH_PRECOMPILED_HEADERS */
-
 /* Qt includes: */
-# include <QSet>
+#include <QSet>
 
 /* GUI includes: */
-# include "UIMediumEnumerator.h"
-# include "UIThreadPool.h"
-# include "UIVirtualBoxEventHandler.h"
-# include "VBoxGlobal.h"
+#include "UIMediumEnumerator.h"
+#include "UIThreadPool.h"
+#include "UIVirtualBoxEventHandler.h"
+#include "VBoxGlobal.h"
 
 /* COM includes: */
-# include "COMEnums.h"
-# include "CMachine.h"
-# include "CSnapshot.h"
-# include "CMediumAttachment.h"
+#include "COMEnums.h"
+#include "CMachine.h"
+#include "CSnapshot.h"
+#include "CMediumAttachment.h"
 
-#endif /* !VBOX_WITH_PRECOMPILED_HEADERS */
+
+template<class T>
+static QStringList toStringList(const QList<T> &list)
+{
+    QStringList l;
+    foreach(const T &t, list)
+        l << t.toString();
+    return l;
+}
 
 
 /** UITask extension used for medium enumeration purposes. */
@@ -85,17 +89,17 @@ UIMediumEnumerator::UIMediumEnumerator()
     connect(vboxGlobal().threadPool(), &UIThreadPool::sigTaskComplete, this, &UIMediumEnumerator::sltHandleMediumEnumerationTaskComplete);
 }
 
-QList<QString> UIMediumEnumerator::mediumIDs() const
+QList<QUuid> UIMediumEnumerator::mediumIDs() const
 {
     /* Return keys of current medium-map: */
-    return m_mediums.keys();
+    return m_media.keys();
 }
 
-UIMedium UIMediumEnumerator::medium(const QString &strMediumID)
+UIMedium UIMediumEnumerator::medium(const QUuid &uMediumID)
 {
     /* Search through current medium-map for the medium with passed ID: */
-    if (m_mediums.contains(strMediumID))
-        return m_mediums[strMediumID];
+    if (m_media.contains(uMediumID))
+        return m_media[uMediumID];
     /* Return NULL medium otherwise: */
     return UIMedium();
 }
@@ -103,55 +107,65 @@ UIMedium UIMediumEnumerator::medium(const QString &strMediumID)
 void UIMediumEnumerator::createMedium(const UIMedium &medium)
 {
     /* Get medium ID: */
-    const QString strMediumID = medium.id();
+    const QUuid uMediumID = medium.id();
 
     /* Do not create UIMedium(s) with incorrect ID: */
-    AssertReturnVoid(!strMediumID.isNull());
-    AssertReturnVoid(strMediumID != UIMedium::nullID());
+    AssertReturnVoid(!uMediumID.isNull());
+    AssertReturnVoid(uMediumID != UIMedium::nullID());
     /* Make sure medium doesn't exists already: */
-    AssertReturnVoid(!m_mediums.contains(strMediumID));
+    AssertReturnVoid(!m_media.contains(uMediumID));
 
     /* Insert medium: */
-    m_mediums[strMediumID] = medium;
-    LogRel(("GUI: UIMediumEnumerator: Medium with key={%s} created\n", strMediumID.toUtf8().constData()));
+    m_media[uMediumID] = medium;
+    LogRel(("GUI: UIMediumEnumerator: Medium with key={%s} created\n", uMediumID.toString().toUtf8().constData()));
 
     /* Notify listener: */
-    emit sigMediumCreated(strMediumID);
+    emit sigMediumCreated(uMediumID);
 }
 
-void UIMediumEnumerator::deleteMedium(const QString &strMediumID)
+void UIMediumEnumerator::deleteMedium(const QUuid &uMediumID)
 {
     /* Do not delete UIMedium(s) with incorrect ID: */
-    AssertReturnVoid(!strMediumID.isNull());
-    AssertReturnVoid(strMediumID != UIMedium::nullID());
+    AssertReturnVoid(!uMediumID.isNull());
+    AssertReturnVoid(uMediumID != UIMedium::nullID());
     /* Make sure medium still exists: */
-    AssertReturnVoid(m_mediums.contains(strMediumID));
+    AssertReturnVoid(m_media.contains(uMediumID));
 
     /* Remove medium: */
-    m_mediums.remove(strMediumID);
-    LogRel(("GUI: UIMediumEnumerator: Medium with key={%s} deleted\n", strMediumID.toUtf8().constData()));
+    m_media.remove(uMediumID);
+    LogRel(("GUI: UIMediumEnumerator: Medium with key={%s} deleted\n", uMediumID.toString().toUtf8().constData()));
 
     /* Notify listener: */
-    emit sigMediumDeleted(strMediumID);
+    emit sigMediumDeleted(uMediumID);
 }
 
-void UIMediumEnumerator::enumerateMediums()
+void UIMediumEnumerator::enumerateMedia(const CMediumVector &mediaList /* = CMediumVector() */)
 {
     /* Make sure we are not already in progress: */
     AssertReturnVoid(!m_fMediumEnumerationInProgress);
 
-    /* Compose new map of all currently known mediums & their children.
-     * While composing we are using data from already existing mediums. */
-    UIMediumMap mediums;
-    addNullMediumToMap(mediums);
-    addHardDisksToMap(vboxGlobal().virtualBox().GetHardDisks(), mediums);
-    addMediumsToMap(vboxGlobal().host().GetDVDDrives(), mediums, UIMediumType_DVD);
-    addMediumsToMap(vboxGlobal().virtualBox().GetDVDImages(), mediums, UIMediumType_DVD);
-    addMediumsToMap(vboxGlobal().host().GetFloppyDrives(), mediums, UIMediumType_Floppy);
-    addMediumsToMap(vboxGlobal().virtualBox().GetFloppyImages(), mediums, UIMediumType_Floppy);
-    if (vboxGlobal().isCleaningUp())
+    /* Compose new map of all currently known media & their children.
+     * While composing we are using data from already existing media. */
+    UIMediumMap media;
+    addNullMediumToMap(media);
+    /* If @p mediaList is empty we start the media enumeration with all known media: */
+    if (mediaList.isEmpty())
+    {
+        addMediaToMap(vboxGlobal().virtualBox().GetHardDisks(), media);
+        addMediaToMap(vboxGlobal().host().GetDVDDrives(), media);
+        addMediaToMap(vboxGlobal().virtualBox().GetDVDImages(), media);
+        addMediaToMap(vboxGlobal().host().GetFloppyDrives(), media);
+        addMediaToMap(vboxGlobal().virtualBox().GetFloppyImages(), media);
+    }
+    else
+    {
+        addMediaToMap(vboxGlobal().host().GetDVDDrives(), media);
+        addMediaToMap(vboxGlobal().virtualBox().GetDVDImages(), media);
+        addMediaToMap(mediaList, media);
+    }
+    if (VBoxGlobal::isCleaningUp())
         return; /* VBoxGlobal is cleaning up, abort immediately. */
-    m_mediums = mediums;
+    m_media = media;
 
     /* Notify listener: */
     LogRel(("GUI: UIMediumEnumerator: Medium-enumeration started...\n"));
@@ -159,7 +173,7 @@ void UIMediumEnumerator::enumerateMediums()
     emit sigMediumEnumerationStarted();
 
     /* Make sure we really have more than one medium (which is Null): */
-    if (m_mediums.size() == 1)
+    if (m_media.size() == 1)
     {
         /* Notify listener: */
         LogRel(("GUI: UIMediumEnumerator: Medium-enumeration finished!\n"));
@@ -168,38 +182,48 @@ void UIMediumEnumerator::enumerateMediums()
     }
 
     /* Start enumeration for UIMedium(s) with correct ID: */
-    foreach (const QString &strMediumID, m_mediums.keys())
-        if (!strMediumID.isNull() && strMediumID != UIMedium::nullID())
-            createMediumEnumerationTask(m_mediums[strMediumID]);
+    foreach (const QUuid &uMediumID, m_media.keys())
+        if (!uMediumID.isNull() && uMediumID != UIMedium::nullID())
+            createMediumEnumerationTask(m_media[uMediumID]);
 }
 
-void UIMediumEnumerator::sltHandleMachineUpdate(QString strMachineID)
+void UIMediumEnumerator::refreshMedia()
+{
+    /* Make sure we are not already in progress: */
+    AssertReturnVoid(!m_fMediumEnumerationInProgress);
+
+    /* Refresh all known media we have: */
+    foreach (const QUuid &uMediumID, m_media.keys())
+        m_media[uMediumID].refresh();
+}
+
+void UIMediumEnumerator::sltHandleMachineUpdate(const QUuid &uMachineID)
 {
     LogRel2(("GUI: UIMediumEnumerator: Machine (or snapshot) event received, ID = %s\n",
-             strMachineID.toUtf8().constData()));
+             uMachineID.toString().toUtf8().constData()));
 
     /* Gather previously used UIMedium IDs: */
-    QStringList previousUIMediumIDs;
-    calculateCachedUsage(strMachineID, previousUIMediumIDs, true /* take into account current state only */);
+    QList<QUuid> previousUIMediumIDs;
+    calculateCachedUsage(uMachineID, previousUIMediumIDs, true /* take into account current state only */);
     LogRel2(("GUI: UIMediumEnumerator:  Old usage: %s\n",
-             previousUIMediumIDs.isEmpty() ? "<empty>" : previousUIMediumIDs.join(", ").toUtf8().constData()));
+             previousUIMediumIDs.isEmpty() ? "<empty>" : toStringList(previousUIMediumIDs).join(", ").toUtf8().constData()));
 
     /* Gather currently used CMediums and their IDs: */
     CMediumMap currentCMediums;
-    QStringList currentCMediumIDs;
-    calculateActualUsage(strMachineID, currentCMediums, currentCMediumIDs, true /* take into account current state only */);
+    QList<QUuid> currentCMediumIDs;
+    calculateActualUsage(uMachineID, currentCMediums, currentCMediumIDs, true /* take into account current state only */);
     LogRel2(("GUI: UIMediumEnumerator:  New usage: %s\n",
-             currentCMediumIDs.isEmpty() ? "<empty>" : currentCMediumIDs.join(", ").toUtf8().constData()));
+             currentCMediumIDs.isEmpty() ? "<empty>" : toStringList(currentCMediumIDs).join(", ").toUtf8().constData()));
 
-    /* Determine excluded mediums: */
-    const QSet<QString> previousSet = previousUIMediumIDs.toSet();
-    const QSet<QString> currentSet = currentCMediumIDs.toSet();
-    const QSet<QString> excludedSet = previousSet - currentSet;
-    const QStringList excludedUIMediumIDs = excludedSet.toList();
+    /* Determine excluded media: */
+    const QSet<QUuid> previousSet = previousUIMediumIDs.toSet();
+    const QSet<QUuid> currentSet = currentCMediumIDs.toSet();
+    const QSet<QUuid> excludedSet = previousSet - currentSet;
+    const QList<QUuid> excludedUIMediumIDs = excludedSet.toList();
     if (!excludedUIMediumIDs.isEmpty())
-        LogRel2(("GUI: UIMediumEnumerator:  Items excluded from usage: %s\n", excludedUIMediumIDs.join(", ").toUtf8().constData()));
+        LogRel2(("GUI: UIMediumEnumerator:  Items excluded from usage: %s\n", toStringList(excludedUIMediumIDs).join(", ").toUtf8().constData()));
     if (!currentCMediumIDs.isEmpty())
-        LogRel2(("GUI: UIMediumEnumerator:  Items currently in usage: %s\n", currentCMediumIDs.join(", ").toUtf8().constData()));
+        LogRel2(("GUI: UIMediumEnumerator:  Items currently in usage: %s\n", toStringList(currentCMediumIDs).join(", ").toUtf8().constData()));
 
     /* Update cache for excluded UIMediums: */
     recacheFromCachedUsage(excludedUIMediumIDs);
@@ -208,24 +232,24 @@ void UIMediumEnumerator::sltHandleMachineUpdate(QString strMachineID)
     recacheFromActualUsage(currentCMediums, currentCMediumIDs);
 
     LogRel2(("GUI: UIMediumEnumerator: Machine (or snapshot) event processed, ID = %s\n",
-             strMachineID.toUtf8().constData()));
+             uMachineID.toString().toUtf8().constData()));
 }
 
-void UIMediumEnumerator::sltHandleMachineRegistration(QString strMachineID, bool fRegistered)
+void UIMediumEnumerator::sltHandleMachineRegistration(const QUuid &uMachineID, const bool fRegistered)
 {
     LogRel2(("GUI: UIMediumEnumerator: Machine %s event received, ID = %s\n",
              fRegistered ? "registration" : "unregistration",
-             strMachineID.toUtf8().constData()));
+             uMachineID.toString().toUtf8().constData()));
 
     /* Machine was registered: */
     if (fRegistered)
     {
         /* Gather currently used CMediums and their IDs: */
         CMediumMap currentCMediums;
-        QStringList currentCMediumIDs;
-        calculateActualUsage(strMachineID, currentCMediums, currentCMediumIDs, false /* take into account current state only */);
+        QList<QUuid> currentCMediumIDs;
+        calculateActualUsage(uMachineID, currentCMediums, currentCMediumIDs, false /* take into account current state only */);
         LogRel2(("GUI: UIMediumEnumerator:  New usage: %s\n",
-                 currentCMediumIDs.isEmpty() ? "<empty>" : currentCMediumIDs.join(", ").toUtf8().constData()));
+                 currentCMediumIDs.isEmpty() ? "<empty>" : toStringList(currentCMediumIDs).join(", ").toUtf8().constData()));
         /* Update cache with currently used CMediums: */
         recacheFromActualUsage(currentCMediums, currentCMediumIDs);
     }
@@ -233,43 +257,43 @@ void UIMediumEnumerator::sltHandleMachineRegistration(QString strMachineID, bool
     else
     {
         /* Gather previously used UIMedium IDs: */
-        QStringList previousUIMediumIDs;
-        calculateCachedUsage(strMachineID, previousUIMediumIDs, false /* take into account current state only */);
+        QList<QUuid> previousUIMediumIDs;
+        calculateCachedUsage(uMachineID, previousUIMediumIDs, false /* take into account current state only */);
         LogRel2(("GUI: UIMediumEnumerator:  Old usage: %s\n",
-                 previousUIMediumIDs.isEmpty() ? "<empty>" : previousUIMediumIDs.join(", ").toUtf8().constData()));
+                 previousUIMediumIDs.isEmpty() ? "<empty>" : toStringList(previousUIMediumIDs).join(", ").toUtf8().constData()));
         /* Update cache for previously used UIMediums: */
         recacheFromCachedUsage(previousUIMediumIDs);
     }
 
     LogRel2(("GUI: UIMediumEnumerator: Machine %s event processed, ID = %s\n",
              fRegistered ? "registration" : "unregistration",
-             strMachineID.toUtf8().constData()));
+             uMachineID.toString().toUtf8().constData()));
 }
 
-void UIMediumEnumerator::sltHandleSnapshotDeleted(QString strMachineID, QString strSnapshotID)
+void UIMediumEnumerator::sltHandleSnapshotDeleted(const QUuid &uMachineID, const QUuid &uSnapshotID)
 {
     LogRel2(("GUI: UIMediumEnumerator: Snapshot-deleted event received, Machine ID = {%s}, Snapshot ID = {%s}\n",
-             strMachineID.toUtf8().constData(), strSnapshotID.toUtf8().constData()));
+             uMachineID.toString().toUtf8().constData(), uSnapshotID.toString().toUtf8().constData()));
 
     /* Gather previously used UIMedium IDs: */
-    QStringList previousUIMediumIDs;
-    calculateCachedUsage(strMachineID, previousUIMediumIDs, false /* take into account current state only */);
+    QList<QUuid> previousUIMediumIDs;
+    calculateCachedUsage(uMachineID, previousUIMediumIDs, false /* take into account current state only */);
     LogRel2(("GUI: UIMediumEnumerator:  Old usage: %s\n",
-             previousUIMediumIDs.isEmpty() ? "<empty>" : previousUIMediumIDs.join(", ").toUtf8().constData()));
+             previousUIMediumIDs.isEmpty() ? "<empty>" : toStringList(previousUIMediumIDs).join(", ").toUtf8().constData()));
 
     /* Gather currently used CMediums and their IDs: */
     CMediumMap currentCMediums;
-    QStringList currentCMediumIDs;
-    calculateActualUsage(strMachineID, currentCMediums, currentCMediumIDs, true /* take into account current state only */);
+    QList<QUuid> currentCMediumIDs;
+    calculateActualUsage(uMachineID, currentCMediums, currentCMediumIDs, true /* take into account current state only */);
     LogRel2(("GUI: UIMediumEnumerator:  New usage: %s\n",
-             currentCMediumIDs.isEmpty() ? "<empty>" : currentCMediumIDs.join(", ").toUtf8().constData()));
+             currentCMediumIDs.isEmpty() ? "<empty>" : toStringList(currentCMediumIDs).join(", ").toUtf8().constData()));
 
     /* Update everything: */
     recacheFromCachedUsage(previousUIMediumIDs);
     recacheFromActualUsage(currentCMediums, currentCMediumIDs);
 
     LogRel2(("GUI: UIMediumEnumerator: Snapshot-deleted event processed, Machine ID = {%s}, Snapshot ID = {%s}\n",
-             strMachineID.toUtf8().constData(), strSnapshotID.toUtf8().constData()));
+             uMachineID.toString().toUtf8().constData(), uSnapshotID.toString().toUtf8().constData()));
 }
 
 void UIMediumEnumerator::sltHandleMediumEnumerationTaskComplete(UITask *pTask)
@@ -281,54 +305,54 @@ void UIMediumEnumerator::sltHandleMediumEnumerationTaskComplete(UITask *pTask)
 
     /* Get enumerated UIMedium: */
     const UIMedium uimedium = pTask->property("medium").value<UIMedium>();
-    const QString strUIMediumKey = uimedium.key();
-    LogRel2(("GUI: UIMediumEnumerator: Medium with key={%s} enumerated\n", strUIMediumKey.toUtf8().constData()));
+    const QUuid uUIMediumKey = uimedium.key();
+    LogRel2(("GUI: UIMediumEnumerator: Medium with key={%s} enumerated\n", uUIMediumKey.toString().toUtf8().constData()));
 
     /* Remove task from internal set: */
     m_tasks.remove(pTask);
 
     /* Make sure such UIMedium still exists: */
-    if (!m_mediums.contains(strUIMediumKey))
+    if (!m_media.contains(uUIMediumKey))
     {
-        LogRel2(("GUI: UIMediumEnumerator: Medium with key={%s} already deleted by a third party\n", strUIMediumKey.toUtf8().constData()));
+        LogRel2(("GUI: UIMediumEnumerator: Medium with key={%s} already deleted by a third party\n", uUIMediumKey.toString().toUtf8().constData()));
         return;
     }
 
     /* Check if UIMedium ID was changed: */
-    const QString strUIMediumID = uimedium.id();
+    const QUuid uUIMediumID = uimedium.id();
     /* UIMedium ID was changed to nullID: */
-    if (strUIMediumID == UIMedium::nullID())
+    if (uUIMediumID == UIMedium::nullID())
     {
         /* Delete this medium: */
-        m_mediums.remove(strUIMediumKey);
-        LogRel2(("GUI: UIMediumEnumerator: Medium with key={%s} closed and deleted (after enumeration)\n", strUIMediumKey.toUtf8().constData()));
+        m_media.remove(uUIMediumKey);
+        LogRel2(("GUI: UIMediumEnumerator: Medium with key={%s} closed and deleted (after enumeration)\n", uUIMediumKey.toString().toUtf8().constData()));
 
         /* And notify listener about delete: */
-        emit sigMediumDeleted(strUIMediumKey);
+        emit sigMediumDeleted(uUIMediumKey);
     }
     /* UIMedium ID was changed to something proper: */
-    else if (strUIMediumID != strUIMediumKey)
+    else if (uUIMediumID != uUIMediumKey)
     {
         /* We have to reinject enumerated medium: */
-        m_mediums.remove(strUIMediumKey);
-        m_mediums[strUIMediumID] = uimedium;
-        m_mediums[strUIMediumID].setKey(strUIMediumID);
-        LogRel2(("GUI: UIMediumEnumerator: Medium with key={%s} has it changed to {%s}\n", strUIMediumKey.toUtf8().constData(),
-                                                                                           strUIMediumID.toUtf8().constData()));
+        m_media.remove(uUIMediumKey);
+        m_media[uUIMediumID] = uimedium;
+        m_media[uUIMediumID].setKey(uUIMediumID);
+        LogRel2(("GUI: UIMediumEnumerator: Medium with key={%s} has it changed to {%s}\n", uUIMediumKey.toString().toUtf8().constData(),
+                                                                                           uUIMediumID.toString().toUtf8().constData()));
 
         /* And notify listener about delete/create: */
-        emit sigMediumDeleted(strUIMediumKey);
-        emit sigMediumCreated(strUIMediumID);
+        emit sigMediumDeleted(uUIMediumKey);
+        emit sigMediumCreated(uUIMediumID);
     }
     /* UIMedium ID was not changed: */
     else
     {
         /* Just update enumerated medium: */
-        m_mediums[strUIMediumID] = uimedium;
-        LogRel2(("GUI: UIMediumEnumerator: Medium with key={%s} updated\n", strUIMediumID.toUtf8().constData()));
+        m_media[uUIMediumID] = uimedium;
+        LogRel2(("GUI: UIMediumEnumerator: Medium with key={%s} updated\n", uUIMediumID.toString().toUtf8().constData()));
 
         /* And notify listener about update: */
-        emit sigMediumEnumerated(strUIMediumID);
+        emit sigMediumEnumerated(uUIMediumID);
     }
 
     /* If there are no more tasks we know about: */
@@ -344,8 +368,8 @@ void UIMediumEnumerator::sltHandleMediumEnumerationTaskComplete(UITask *pTask)
 void UIMediumEnumerator::retranslateUi()
 {
     /* Translating NULL uimedium by recreating it: */
-    if (m_mediums.contains(UIMedium::nullID()))
-        m_mediums[UIMedium::nullID()] = UIMedium();
+    if (m_media.contains(UIMedium::nullID()))
+        m_media[UIMedium::nullID()] = UIMedium();
 }
 
 void UIMediumEnumerator::createMediumEnumerationTask(const UIMedium &medium)
@@ -358,92 +382,72 @@ void UIMediumEnumerator::createMediumEnumerationTask(const UIMedium &medium)
     vboxGlobal().threadPool()->enqueueTask(pTask);
 }
 
-void UIMediumEnumerator::addNullMediumToMap(UIMediumMap &mediums)
+void UIMediumEnumerator::addNullMediumToMap(UIMediumMap &media)
 {
     /* Insert NULL uimedium to the passed uimedium map.
      * Get existing one from the previous map if any. */
-    QString strNullMediumID = UIMedium::nullID();
-    UIMedium uimedium = m_mediums.contains(strNullMediumID) ? m_mediums[strNullMediumID] : UIMedium();
-    mediums.insert(strNullMediumID, uimedium);
+    QUuid uNullMediumID = UIMedium::nullID();
+    UIMedium uimedium = m_media.contains(uNullMediumID) ? m_media[uNullMediumID] : UIMedium();
+    media.insert(uNullMediumID, uimedium);
 }
 
-void UIMediumEnumerator::addMediumsToMap(const CMediumVector &inputMediums, UIMediumMap &outputMediums, UIMediumType mediumType)
+void UIMediumEnumerator::addMediaToMap(const CMediumVector &inputMedia, UIMediumMap &outputMedia)
 {
     /* Insert hard-disks to the passed uimedium map.
      * Get existing one from the previous map if any. */
-    foreach (CMedium medium, inputMediums)
+    foreach (CMedium medium, inputMedia)
     {
         /* If VBoxGlobal is cleaning up, abort immediately: */
-        if (vboxGlobal().isCleaningUp())
+        if (VBoxGlobal::isCleaningUp())
             break;
 
         /* Prepare uimedium on the basis of current medium: */
-        QString strMediumID = medium.GetId();
-        UIMedium uimedium = m_mediums.contains(strMediumID) ? m_mediums[strMediumID] :
-                                                              UIMedium(medium, mediumType);
+        QUuid uMediumID = medium.GetId();
+        UIMedium uimedium = m_media.contains(uMediumID) ? m_media[uMediumID] :
+            UIMedium(medium, UIMediumDefs::mediumTypeToLocal(medium.GetDeviceType()));
 
         /* Insert uimedium into map: */
-        outputMediums.insert(uimedium.id(), uimedium);
-    }
-}
-
-void UIMediumEnumerator::addHardDisksToMap(const CMediumVector &inputMediums, UIMediumMap &outputMediums)
-{
-    /* Insert hard-disks to the passed uimedium map.
-     * Get existing one from the previous map if any. */
-    foreach (CMedium medium, inputMediums)
-    {
-        /* If VBoxGlobal is cleaning up, abort immediately: */
-        if (vboxGlobal().isCleaningUp())
-            break;
-
-        /* Prepare uimedium on the basis of current medium: */
-        QString strMediumID = medium.GetId();
-        UIMedium uimedium = m_mediums.contains(strMediumID) ? m_mediums[strMediumID] :
-                                                              UIMedium(medium, UIMediumType_HardDisk);
-
-        /* Insert uimedium into map: */
-        outputMediums.insert(uimedium.id(), uimedium);
+        outputMedia.insert(uimedium.id(), uimedium);
 
         /* Insert medium children into map too: */
-        addHardDisksToMap(medium.GetChildren(), outputMediums);
+        addMediaToMap(medium.GetChildren(), outputMedia);
     }
 }
 
 /**
  * Calculates last known UIMedium <i>usage</i> based on cached data.
- * @param strMachineID describes the machine we are calculating <i>usage</i> for.
+ * @param uMachineID describes the machine we are calculating <i>usage</i> for.
  * @param previousUIMediumIDs receives UIMedium IDs used in cached data.
  * @param fTakeIntoAccountCurrentStateOnly defines whether we should take into accound current VM state only.
  */
-void UIMediumEnumerator::calculateCachedUsage(const QString &strMachineID, QStringList &previousUIMediumIDs, bool fTakeIntoAccountCurrentStateOnly) const
+void UIMediumEnumerator::calculateCachedUsage(const QUuid &uMachineID, QList<QUuid> &previousUIMediumIDs, const bool fTakeIntoAccountCurrentStateOnly) const
 {
     /* For each the UIMedium ID cache have: */
-    foreach (const QString &strMediumID, mediumIDs())
+    foreach (const QUuid &uMediumID, mediumIDs())
     {
         /* Get corresponding UIMedium: */
-        const UIMedium &uimedium = m_mediums[strMediumID];
+        const UIMedium &uimedium = m_media[uMediumID];
         /* Get the list of the machines this UIMedium attached to.
          * Take into account current-state only if necessary. */
-        const QList<QString> &machineIDs = fTakeIntoAccountCurrentStateOnly ?
+        const QList<QUuid> &machineIDs = fTakeIntoAccountCurrentStateOnly ?
                                            uimedium.curStateMachineIds() : uimedium.machineIds();
         /* Add this UIMedium ID to previous usage if necessary: */
-        if (machineIDs.contains(strMachineID))
-            previousUIMediumIDs << strMediumID;
+        if (machineIDs.contains(uMachineID))
+            previousUIMediumIDs.append(uMediumID);
     }
 }
 
 /**
  * Calculates new CMedium <i>usage</i> based on actual data.
- * @param strMachineID describes the machine we are calculating <i>usage</i> for.
+ * @param uMachineID describes the machine we are calculating <i>usage</i> for.
  * @param currentCMediums receives CMedium used in actual data.
  * @param currentCMediumIDs receives CMedium IDs used in actual data.
  * @param fTakeIntoAccountCurrentStateOnly defines whether we should take into accound current VM state only.
  */
-void UIMediumEnumerator::calculateActualUsage(const QString &strMachineID, CMediumMap &currentCMediums, QStringList &currentCMediumIDs, bool fTakeIntoAccountCurrentStateOnly) const
+void UIMediumEnumerator::calculateActualUsage(const QUuid &uMachineID, CMediumMap &currentCMediums, QList<QUuid> &currentCMediumIDs, const bool fTakeIntoAccountCurrentStateOnly) const
 {
     /* Search for corresponding machine: */
-    CMachine machine = vboxGlobal().virtualBox().FindMachine(strMachineID);
+    CMachine machine = vboxGlobal().virtualBox().FindMachine(uMachineID.toString());
     if (machine.isNull())
     {
         /* Usually means the machine is already gone, not harmful. */
@@ -463,7 +467,7 @@ void UIMediumEnumerator::calculateActualUsage(const QString &strMachineID, CMedi
  * @param currentCMediums receives CMedium used in actual data.
  * @param currentCMediumIDs receives CMedium IDs used in actual data.
  */
-void UIMediumEnumerator::calculateActualUsage(const CSnapshot &snapshot, CMediumMap &currentCMediums, QStringList &currentCMediumIDs) const
+void UIMediumEnumerator::calculateActualUsage(const CSnapshot &snapshot, CMediumMap &currentCMediums, QList<QUuid> &currentCMediumIDs) const
 {
     /* Check passed snapshot: */
     if (snapshot.isNull())
@@ -483,7 +487,7 @@ void UIMediumEnumerator::calculateActualUsage(const CSnapshot &snapshot, CMedium
  * @param currentCMediums receives CMedium used in actual data.
  * @param currentCMediumIDs receives CMedium IDs used in actual data.
  */
-void UIMediumEnumerator::calculateActualUsage(const CMachine &machine, CMediumMap &currentCMediums, QStringList &currentCMediumIDs) const
+void UIMediumEnumerator::calculateActualUsage(const CMachine &machine, CMediumMap &currentCMediums, QList<QUuid> &currentCMediumIDs) const
 {
     /* Check passed machine: */
     AssertReturnVoid(!machine.isNull());
@@ -496,12 +500,12 @@ void UIMediumEnumerator::calculateActualUsage(const CMachine &machine, CMediumMa
         if (!cmedium.isNull())
         {
             /* Make sure that CMedium was not yet closed: */
-            const QString strCMediumID = cmedium.GetId();
-            if (cmedium.isOk() && !strCMediumID.isNull())
+            const QUuid uCMediumID = cmedium.GetId();
+            if (cmedium.isOk() && !uCMediumID.isNull())
             {
                 /* Add this CMedium to current usage: */
-                currentCMediums.insert(strCMediumID, cmedium);
-                currentCMediumIDs << strCMediumID;
+                currentCMediums.insert(uCMediumID, cmedium);
+                currentCMediumIDs.append(uCMediumID);
             }
         }
     }
@@ -511,16 +515,16 @@ void UIMediumEnumerator::calculateActualUsage(const CMachine &machine, CMediumMa
  * Updates cache using known changes in cached data.
  * @param previousUIMediumIDs reflects UIMedium IDs used in cached data.
  */
-void UIMediumEnumerator::recacheFromCachedUsage(const QStringList &previousUIMediumIDs)
+void UIMediumEnumerator::recacheFromCachedUsage(const QList<QUuid> &previousUIMediumIDs)
 {
     /* For each of previously used UIMedium ID: */
-    foreach (const QString &strMediumID, previousUIMediumIDs)
+    foreach (const QUuid &uMediumID, previousUIMediumIDs)
     {
         /* Make sure this ID still in our map: */
-        if (m_mediums.contains(strMediumID))
+        if (m_media.contains(uMediumID))
         {
             /* Get corresponding UIMedium: */
-            UIMedium &uimedium = m_mediums[strMediumID];
+            UIMedium &uimedium = m_media[uMediumID];
 
             /* If corresponding CMedium still exists: */
             CMedium cmedium = uimedium.medium();
@@ -535,11 +539,11 @@ void UIMediumEnumerator::recacheFromCachedUsage(const QStringList &previousUIMed
             else
             {
                 /* Uncache corresponding UIMedium: */
-                m_mediums.remove(strMediumID);
-                LogRel2(("GUI: UIMediumEnumerator:  Medium with key={%s} uncached\n", strMediumID.toUtf8().constData()));
+                m_media.remove(uMediumID);
+                LogRel2(("GUI: UIMediumEnumerator:  Medium with key={%s} uncached\n", uMediumID.toString().toUtf8().constData()));
 
                 /* And notify listeners: */
-                emit sigMediumDeleted(strMediumID);
+                emit sigMediumDeleted(uMediumID);
             }
         }
     }
@@ -550,32 +554,31 @@ void UIMediumEnumerator::recacheFromCachedUsage(const QStringList &previousUIMed
  * @param currentCMediums reflects CMedium used in actual data.
  * @param currentCMediumIDs reflects CMedium IDs used in actual data.
  */
-void UIMediumEnumerator::recacheFromActualUsage(const CMediumMap &currentCMediums, const QStringList &currentCMediumIDs)
+void UIMediumEnumerator::recacheFromActualUsage(const CMediumMap &currentCMediums, const QList<QUuid> &currentCMediumIDs)
 {
     /* For each of currently used CMedium ID: */
-    foreach (const QString &strCMediumID, currentCMediumIDs)
+    foreach (const QUuid &uCMediumID, currentCMediumIDs)
     {
         /* If that ID is not in our map: */
-        if (!m_mediums.contains(strCMediumID))
+        if (!m_media.contains(uCMediumID))
         {
             /* Create new UIMedium: */
-            const CMedium &cmedium = currentCMediums[strCMediumID];
+            const CMedium &cmedium = currentCMediums[uCMediumID];
             UIMedium uimedium(cmedium, UIMediumDefs::mediumTypeToLocal(cmedium.GetDeviceType()));
-            QString strUIMediumKey = uimedium.key();
+            QUuid uUIMediumKey = uimedium.key();
 
             /* Cache created UIMedium: */
-            m_mediums.insert(strUIMediumKey, uimedium);
-            LogRel2(("GUI: UIMediumEnumerator:  Medium with key={%s} cached\n", strUIMediumKey.toUtf8().constData()));
+            m_media.insert(uUIMediumKey, uimedium);
+            LogRel2(("GUI: UIMediumEnumerator:  Medium with key={%s} cached\n", uUIMediumKey.toString().toUtf8().constData()));
 
             /* And notify listeners: */
-            emit sigMediumCreated(strUIMediumKey);
+            emit sigMediumCreated(uUIMediumKey);
         }
 
         /* Enumerate corresponding UIMedium: */
-        createMediumEnumerationTask(m_mediums[strCMediumID]);
+        createMediumEnumerationTask(m_media[uCMediumID]);
     }
 }
 
 
 #include "UIMediumEnumerator.moc"
-
