@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2011-2016 Oracle Corporation
+ * Copyright (C) 2011-2019 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -15,8 +15,11 @@
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
  */
 
-#ifndef ___UINetworkDefs_h___
-#define ___UINetworkDefs_h___
+#ifndef FEQT_INCLUDED_SRC_net_UINetworkDefs_h
+#define FEQT_INCLUDED_SRC_net_UINetworkDefs_h
+#ifndef RT_WITHOUT_PRAGMA_ONCE
+# pragma once
+#endif
 
 /* Qt includes: */
 #include <QMap>
@@ -31,5 +34,5 @@ enum UINetworkRequestType
 /** User dictionary. */
 typedef QMap<QString, QString> UserDictionary;
 
-#endif /* !___UINetworkDefs_h___ */
+#endif /* !FEQT_INCLUDED_SRC_net_UINetworkDefs_h */
 
