@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2010-2017 Oracle Corporation
+ * Copyright (C) 2010-2019 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -15,8 +15,11 @@
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
  */
 
-#ifndef ___UIMachineViewSeamless_h___
-#define ___UIMachineViewSeamless_h___
+#ifndef FEQT_INCLUDED_SRC_runtime_seamless_UIMachineViewSeamless_h
+#define FEQT_INCLUDED_SRC_runtime_seamless_UIMachineViewSeamless_h
+#ifndef RT_WITHOUT_PRAGMA_ONCE
+# pragma once
+#endif
 
 /* Local includes */
 #include "UIMachineView.h"
@@ -73,5 +76,5 @@ private:
     friend class UIMachineView;
 };
 
-#endif // !___UIMachineViewSeamless_h___
+#endif /* !FEQT_INCLUDED_SRC_runtime_seamless_UIMachineViewSeamless_h */
 
