@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2006-2016 Oracle Corporation
+ * Copyright (C) 2006-2019 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -15,6 +15,7 @@
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
  */
 
+#define LOG_GROUP LOG_GROUP_MAIN_USBPROXYBACKEND
 #include "USBProxyService.h"
 #include "HostUSBDeviceImpl.h"
 #include "HostImpl.h"
@@ -22,10 +23,10 @@
 #include "VirtualBoxImpl.h"
 
 #include "AutoCaller.h"
-#include "Logging.h"
+#include "LoggingNew.h"
 
 #include <VBox/com/array.h>
-#include <VBox/err.h>
+#include <iprt/errcore.h>
 #include <iprt/asm.h>
 #include <iprt/semaphore.h>
 #include <iprt/thread.h>
@@ -264,6 +265,7 @@ HRESULT USBProxyService::removeUSBDeviceSource(const com::Utf8Str &aId)
  *
  * @param   aMachine        The machine to attach the device to.
  * @param   aId             The UUID of the USB device to capture and attach.
+ * @param   aCaptureFilename
  *
  * @returns COM status code and error info.
  *
@@ -423,6 +425,8 @@ HRESULT USBProxyService::autoCaptureDevicesForVM(SessionMachine *aMachine)
  * to other VMs.
  *
  * @param   aMachine        The machine to detach devices from.
+ * @param   aDone
+ * @param   aAbnormal
  *
  * @returns COM status code, perhaps with error info.
  *
@@ -541,7 +545,7 @@ HRESULT USBProxyService::i_saveSettings(settings::USBDeviceSourcesList &llUSBDev
  * VM attaching. This may result in IPC and temporary lock abandonment.
  *
  * @param   aDevice     The device in question.
- * @param   aUSBDevice  The USB device structure.
+ * @param   pDev        The USB device structure.
  */
 void USBProxyService::i_deviceAdded(ComObjPtr<HostUSBDevice> &aDevice,
                                     PUSBDEVICE pDev)
@@ -574,7 +578,7 @@ void USBProxyService::i_deviceAdded(ComObjPtr<HostUSBDevice> &aDevice,
             && pHostDevice->i_compare(pDev) < 0)
             break;
 
-        it++;
+        ++it;
     }
 
     mDevices.insert(it, aDevice);
@@ -709,6 +713,7 @@ void USBProxyService::deviceChanged(ComObjPtr<HostUSBDevice> &aDevice, bool fRun
  *
  * @returns COM status code (only parameter & state checks will fail).
  * @param   aDevice         The USB device to apply filters to.
+ * @param   llOpenedMachines The list of opened machines.
  * @param   aIgnoreMachine  The machine to ignore filters from (we've just
  *                          detached the device from this machine).
  *
@@ -944,11 +949,11 @@ HRESULT USBProxyService::setError(HRESULT aResultCode, const char *aText, ...)
     va_list va;
     va_start(va, aText);
     HRESULT rc = VirtualBoxBase::setErrorInternal(aResultCode,
-                                                    COM_IIDOF(IHost),
-                                                    "USBProxyService",
-                                                    Utf8Str(aText, va),
-                                                    false /* aWarning*/,
-                                                    true /* aLogIt*/);
+                                                  COM_IIDOF(IHost),
+                                                  "USBProxyService",
+                                                  Utf8Str(aText, va),
+                                                  false /* aWarning*/,
+                                                  true /* aLogIt*/);
     va_end(va);
     return rc;
 }
