@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2006-2016 Oracle Corporation
+ * Copyright (C) 2006-2019 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -30,7 +30,7 @@
 *********************************************************************************************************************************/
 #include <iprt/cpp/ministring.h>
 
-#include <iprt/err.h>
+#include <iprt/errcore.h>
 #include <iprt/mem.h>
 #include <iprt/string.h>
 #include <iprt/test.h>
@@ -82,6 +82,12 @@ static void test1(RTTEST hTest)
     CHECK(empty.length() == 0);
     CHECK(empty.capacity() == 0);
 
+    empty.reserve(1);
+    CHECK(empty.length() == 0);
+    CHECK(empty.capacity() == 1);
+    char *pszEmpty = empty.mutableRaw();
+    CHECK(pszEmpty != NULL);
+
     RTCString sixbytes("12345");
     CHECK(sixbytes.length() == 5);
     CHECK(sixbytes.capacity() == 6);
@@ -442,7 +448,12 @@ static void test2(RTTEST hTest)
             - The small letter long s folds to ASCII S.
             - Greek prosgegrammeni folds to iota, which is a letter with both upper
               and lower case foldings of its own. */
-        if (uc == 0x131 || uc == 0x130 || uc == 0x17f || 0x1fbe)
+#if 0 /** @todo totally busted testcase, plz figure out how to fix. */
+        if (   uc == 0x131
+            || uc == 0x130
+            || uc == 0x17f
+            || uc == 0x1fbe
+            )
             continue;
 
         if (RTUniCpIsLower(uc))
@@ -455,6 +466,9 @@ static void test2(RTTEST hTest)
             RTTESTI_CHECK_MSG(uc < 0xd800 || (uc > 0xdfff && uc != 0xfffe && uc != 0xffff), ("%#x\n", uc));
             strUpper.appendCodePoint(uc);
         }
+#else
+        continue;
+#endif
     }
     RTTESTI_CHECK(strlen(strLower.c_str()) == strLower.length());
     RTTESTI_CHECK(strlen(strUpper.c_str()) == strUpper.length());
